@@ -1,16 +1,20 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #import "OFObject.h"
@@ -21,21 +25,24 @@ OF_ASSUME_NONNULL_BEGIN
 /** @file */
 
 #ifdef OF_HAVE_FILES
-# if (defined(OF_HAVE_CHMOD) && !defined(OF_AMIGAOS)) || defined(DOXYGEN)
+# if (defined(OF_HAVE_CHMOD) && !defined(OF_AMIGAOS) && \
+    !defined(OF_NINTENDO_DS)) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_PERMISSIONS
 # endif
 # if (defined(OF_HAVE_CHOWN) && !defined(OF_AMIGAOS)) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_OWNER
 # endif
-# if (defined(OF_HAVE_LINK) && !defined(OF_AMIGAOS)) || defined(OF_WINDOWS) || \
-    defined(DOXYGEN)
+# if (defined(OF_HAVE_LINK) && !defined(OF_AMIGAOS) && \
+    !defined(OF_NINTENDO_DS)) || defined(OF_WINDOWS) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_LINKS
 # endif
-# if (defined(OF_HAVE_SYMLINK) && !defined(OF_AMIGAOS)) || \
-    defined(OF_WINDOWS) || defined(DOXYGEN)
+# if (defined(OF_HAVE_SYMLINK) && !defined(OF_AMIGAOS) && \
+    !defined(OF_NINTENDO_DS)) || defined(OF_WINDOWS) || defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_SYMLINKS
 # endif
-# if defined(OF_LINUX) || defined(OF_MACOS) || defined(DOXYGEN)
+# if defined(OF_LINUX) || defined(OF_MACOS) || defined(OF_FREEBSD) || \
+    defined(OF_NETBSD) || defined(OF_HAIKU) || defined(OF_SOLARIS) || \
+    defined(DOXYGEN)
 #  define OF_FILE_MANAGER_SUPPORTS_EXTENDED_ATTRIBUTES
 # endif
 #endif
@@ -258,7 +265,7 @@ extern const OFFileAttributeType OFFileTypeUnknown;
 #endif
 
 /**
- * @class OFFileManager OFFileManager.h ObjFW/OFFileManager.h
+ * @class OFFileManager OFFileManager.h ObjFW/ObjFW.h
  *
  * @brief A class which provides management for files, e.g. reading contents of
  *	  directories, deleting files, renaming files, etc.
@@ -695,6 +702,8 @@ OF_SUBCLASSING_RESTRICTED
  *
  * @param name The name of the extended attribute
  * @param path The path of the item to return the extended attribute from
+ * @return The extended attribute data for the specified name of the item at
+ *	   the specified IRI
  * @throw OFGetItemAttributesFailedException Getting the extended attribute
  *					     failed
  * @throw OFNotImplementedException Getting extended attributes is not
@@ -702,6 +711,30 @@ OF_SUBCLASSING_RESTRICTED
  */
 - (OFData *)extendedAttributeDataForName: (OFString *)name
 			    ofItemAtPath: (OFString *)path;
+
+/**
+ * @brief Gets the extended attribute data and type for the specified name
+ *	  of the item at the specified path.
+ *
+ * This method is not available on some systems.
+ *
+ * @param data A pointer to `OFData *` that gets set to the data of the
+ *	       extended attribute
+ * @param type A pointer to `id` that gets set to the type of the extended
+ *	       attribute, if not `NULL`. Gets set to `nil` if the extended
+ *	       attribute has no type. The type of the type depends on the
+ *	       system.
+ * @param name The name of the extended attribute
+ * @param path The path of the item to return the extended attribute from
+ * @throw OFGetItemAttributesFailedException Getting the extended attribute
+ *					     failed
+ * @throw OFNotImplementedException Getting extended attributes is not
+ *				    implemented for the specified item
+ */
+- (void)getExtendedAttributeData: (OFData *_Nonnull *_Nonnull)data
+			 andType: (id _Nullable *_Nullable)type
+			 forName: (OFString *)name
+		    ofItemAtPath: (OFString *)path;
 #endif
 
 /**
@@ -712,6 +745,8 @@ OF_SUBCLASSING_RESTRICTED
  *
  * @param name The name of the extended attribute
  * @param IRI The IRI of the item to return the extended attribute from
+ * @return The extended attribute data for the specified name of the item at
+ *	   the specified IRI
  * @throw OFGetItemAttributesFailedException Getting the extended attribute
  *					     failed
  * @throw OFUnsupportedProtocolException No handler is registered for the IRI's
@@ -722,6 +757,32 @@ OF_SUBCLASSING_RESTRICTED
 - (OFData *)extendedAttributeDataForName: (OFString *)name
 			     ofItemAtIRI: (OFIRI *)IRI;
 
+/**
+ * @brief Gets the extended attribute data and type for the specified name
+ *	  of the item at the specified IRI.
+ *
+ * This method is not available for all IRIs.
+ *
+ * @param data A pointer to `OFData *` that gets set to the data of the
+ *	       extended attribute
+ * @param type A pointer to `id` that gets set to the type of the extended
+ *	       attribute, if not `NULL`. Gets set to `nil` if the extended
+ *	       attribute has no type. The type of the type depends on the IRI
+ *	       handler.
+ * @param name The name of the extended attribute
+ * @param IRI The IRI of the item to return the extended attribute from
+ * @throw OFGetItemAttributesFailedException Getting the extended attribute
+ *					     failed
+ * @throw OFUnsupportedProtocolException No handler is registered for the IRI's
+ *					 scheme
+ * @throw OFNotImplementedException Getting extended attributes is not
+ *				    implemented for the specified item
+ */
+- (void)getExtendedAttributeData: (OFData *_Nonnull *_Nonnull)data
+			 andType: (id _Nullable *_Nullable)type
+			 forName: (OFString *)name
+		     ofItemAtIRI: (OFIRI *)IRI;
+
 #ifdef OF_FILE_MANAGER_SUPPORTS_EXTENDED_ATTRIBUTES
 /**
  * @brief Sets the extended attribute data for the specified name of the item
@@ -740,6 +801,30 @@ OF_SUBCLASSING_RESTRICTED
 - (void)setExtendedAttributeData: (OFData *)data
 			 forName: (OFString *)name
 		    ofItemAtPath: (OFString *)path;
+
+/**
+ * @brief Sets the extended attribute data for the specified name of the item
+ *	  at the specified path.
+ *
+ * This method is not available on some systems.
+ *
+ * @param data The data for the extended attribute
+ * @param type The type for the extended attribute. `nil` does not mean to keep
+ *	       the existing type, but to set it to no type. The type of the
+ *	       type depends on the system.
+ * @param name The name of the extended attribute
+ * @param path The path of the item to set the extended attribute on
+ * @throw OFSetItemAttributesFailedException Setting the extended attribute
+ *					     failed
+ * @throw OFNotImplementedException Setting extended attributes is not
+ *				    implemented for the specified item or a
+ *				    type was specified and typed extended
+ *				    attributes are not supported
+ */
+- (void)setExtendedAttributeData: (OFData *)data
+			 andType: (nullable id)type
+			 forName: (OFString *)name
+		    ofItemAtPath: (OFString *)path;
 #endif
 
 /**
@@ -762,6 +847,32 @@ OF_SUBCLASSING_RESTRICTED
 			 forName: (OFString *)name
 		     ofItemAtIRI: (OFIRI *)IRI;
 
+/**
+ * @brief Sets the extended attribute data for the specified name of the item
+ *	  at the specified IRI.
+ *
+ * This method is not available for all IRIs.
+ *
+ * @param data The data for the extended attribute
+ * @param type The type for the extended attribute. `nil` does not mean to keep
+ *	       the existing type, but to set it to no type. The type of the
+ *	       type depends on the IRI handler.
+ * @param name The name of the extended attribute
+ * @param IRI The IRI of the item to set the extended attribute on
+ * @throw OFSetItemAttributesFailedException Setting the extended attribute
+ *					     failed
+ * @throw OFUnsupportedProtocolException No handler is registered for the IRI's
+ *					 scheme
+ * @throw OFNotImplementedException Setting extended attributes is not
+ *				    implemented for the specified item or a
+ *				    type was specified and typed extended
+ *				    attributes are not supported
+ */
+- (void)setExtendedAttributeData: (OFData *)data
+			 andType: (nullable id)type
+			 forName: (OFString *)name
+		     ofItemAtIRI: (OFIRI *)IRI;
+
 #ifdef OF_FILE_MANAGER_SUPPORTS_EXTENDED_ATTRIBUTES
 /**
  * @brief Removes the extended attribute for the specified name of the item at
