@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -22,9 +22,8 @@
 OF_ASSUME_NONNULL_BEGIN
 
 /**
- * @class OFInvalidServerResponseException \
- *	  OFInvalidServerResponseException.h \
- *	  ObjFW/OFInvalidServerResponseException.h
+ * @class OFInvalidServerResponseException OFInvalidServerResponseException.h
+ *	  ObjFW/ObjFW.h
  *
  * @brief An exception indicating that the server sent an invalid response.
  */
