@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -24,8 +24,8 @@ OF_ASSUME_NONNULL_BEGIN
 @class OFIRI;
 
 /**
- * @class OFRemoveItemFailedException \
- *	  OFRemoveItemFailedException.h ObjFW/OFRemoveItemFailedException.h
+ * @class OFRemoveItemFailedException OFRemoveItemFailedException.h
+ *	  ObjFW/ObjFW.h
  *
  * @brief An exception indicating that removing an item failed.
  */
