@@ -1,16 +1,20 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
- * This file is part of ObjFW. It may be distributed under the terms of the
- * Q Public License 1.0, which can be found in the file LICENSE.QPL included in
- * the packaging of this file.
+ * This program is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License version 3.0 only,
+ * as published by the Free Software Foundation.
  *
- * Alternatively, it may be distributed under the terms of the GNU General
- * Public License, either version 2 or 3, which can be found in the file
- * LICENSE.GPLv2 or LICENSE.GPLv3 respectively included in the packaging of this
- * file.
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
+ * version 3.0 for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public License
+ * version 3.0 along with this program. If not, see
+ * <https://www.gnu.org/licenses/>.
  */
 
 #import "OFStream.h"
@@ -26,6 +30,8 @@ OF_ASSUME_NONNULL_BEGIN
 /**
  * @brief A block which is called when the socket accepted a connection.
  *
+ * @deprecated Use OFStreamSocketAcceptedHandler instead.
+ *
  * @param acceptedSocket The socket which has been accepted
  * @param exception An exception which occurred while accepting the socket or
  *		    `nil` on success
@@ -33,11 +39,25 @@ OF_ASSUME_NONNULL_BEGIN
  *	   connection
  */
 typedef bool (^OFStreamSocketAsyncAcceptBlock)(OFStreamSocket *acceptedSocket,
-    id _Nullable exception);
+    id _Nullable exception)
+    OF_DEPRECATED(ObjFW, 1, 2, "Use OFStreamSocketAcceptedHandler instead");
+
+/**
+ * @brief A handler which is called when the socket accepted a connection.
+ *
+ * @param socket The socket which accepted the connection
+ * @param acceptedSocket The socket which has been accepted
+ * @param exception An exception which occurred while accepting the socket or
+ *		    `nil` on success
+ * @return A bool whether the same handler should be used for the next incoming
+ *	   connection
+ */
+typedef bool (^OFStreamSocketAcceptedHandler)(OFStreamSocket *socket,
+    OFStreamSocket *acceptedSocket, id _Nullable exception);
 #endif
 
 /**
- * @protocol OFStreamSocketDelegate OFStreamSocket.h ObjFW/OFStreamSocket.h
+ * @protocol OFStreamSocketDelegate OFStreamSocket.h ObjFW/ObjFW.h
  *
  * A delegate for OFStreamSocket.
  */
@@ -58,7 +78,7 @@ typedef bool (^OFStreamSocketAsyncAcceptBlock)(OFStreamSocket *acceptedSocket,
 @end
 
 /**
- * @class OFStreamSocket OFStreamSocket.h ObjFW/OFStreamSocket.h
+ * @class OFStreamSocket OFStreamSocket.h ObjFW/ObjFW.h
  *
  * @brief A class which provides methods to create and use stream sockets.
  */
@@ -148,22 +168,49 @@ typedef bool (^OFStreamSocketAsyncAcceptBlock)(OFStreamSocket *acceptedSocket,
 /**
  * @brief Asynchronously accept an incoming connection.
  *
+ * @deprecated Use @ref asyncAcceptWithHandler: instead.
+ *
  * @param block The block to execute when a new connection has been accepted.
  *		Returns whether the next incoming connection should be accepted
  *		by the specified block as well.
  */
-- (void)asyncAcceptWithBlock: (OFStreamSocketAsyncAcceptBlock)block;
+- (void)asyncAcceptWithBlock: (OFStreamSocketAsyncAcceptBlock)block
+    OF_DEPRECATED(ObjFW, 1, 2, "Use -[asyncAcceptWithHandler:] instead");
 
 /**
  * @brief Asynchronously accept an incoming connection.
  *
+ * @param handler The handler to execute when a new connection has been
+ *		  accepted. Returns whether the next incoming connection should
+ *		  be accepted by the specified handler as well.
+ */
+- (void)asyncAcceptWithHandler: (OFStreamSocketAcceptedHandler)handler;
+
+/**
+ * @brief Asynchronously accept an incoming connection.
+ *
+ * @deprecated Use @ref asyncAcceptWithRunLoopMode:handler: instead.
+ *
  * @param runLoopMode The run loop mode in which to perform the async accept
  * @param block The block to execute when a new connection has been accepted.
  *		Returns whether the next incoming connection should be accepted
  *		by the specified block as well.
  */
 - (void)asyncAcceptWithRunLoopMode: (OFRunLoopMode)runLoopMode
-			     block: (OFStreamSocketAsyncAcceptBlock)block;
+			     block: (OFStreamSocketAsyncAcceptBlock)block
+    OF_DEPRECATED(ObjFW, 1, 2,
+	"Use -[asyncAcceptWithRunLoopMode:handler:] instead");
+
+/**
+ * @brief Asynchronously accept an incoming connection.
+ *
+ * @param runLoopMode The run loop mode in which to perform the async accept
+ * @param handler The handler to execute when a new connection has been
+ *		  accepted. Returns whether the next incoming connection
+ *		  should be accepted by the specified handler as well.
+ */
+- (void)asyncAcceptWithRunLoopMode: (OFRunLoopMode)runLoopMode
+			   handler: (OFStreamSocketAcceptedHandler)handler;
 #endif
 
 /**
