@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2008-2024 Jonathan Schleifer <js@nil.im>
+ * Copyright (c) 2008-2025 Jonathan Schleifer <js@nil.im>
  *
  * All rights reserved.
  *
@@ -26,7 +26,7 @@ OF_ASSUME_NONNULL_BEGIN
 @class OFIRI;
 
 /**
- * @class OFSystemInfo OFSystemInfo.h ObjFW/OFSystemInfo.h
+ * @class OFSystemInfo OFSystemInfo.h ObjFW/ObjFW.h
  *
  * @brief A class for querying information about the system.
  */
@@ -94,6 +94,10 @@ OF_SUBCLASSING_RESTRICTED
 # if defined(OF_POWERPC) || defined(OF_POWERPC64) || defined(DOXYGEN)
 @property (class, readonly, nonatomic) bool supportsAltiVec;
 # endif
+# if defined(OF_LOONGARCH64) || defined(DOXYGEN)
+@property (class, readonly, nonatomic) bool supportsLSX;
+@property (class, readonly, nonatomic) bool supportsLASX;
+# endif
 # if defined(OF_WINDOWS) || defined(DOXYGEN)
 @property (class, readonly, nonatomic, getter=isWindowsNT) bool windowsNT;
 # endif
@@ -197,7 +201,7 @@ OF_SUBCLASSING_RESTRICTED
 
 /**
  * @brief Returns a path where temporary files for can be stored.
- * 
+ *
  * If possible, returns a temporary directory for the user, otherwise returns a
  * global temporary directory.
  *
@@ -531,6 +535,26 @@ OF_SUBCLASSING_RESTRICTED
 + (bool)supportsAltiVec;
 #endif
 
+#if defined(OF_LOONGARCH64) || defined(DOXYGEN)
+/**
+ * @brief Returns whether the CPU and OS support LSX.
+ *
+ * @note This method is only available on LoongArch 64!
+ *
+ * @return Whether the CPU and OS support LSX
+ */
++ (bool)supportsLSX;
+
+/**
+ * @brief Returns whether the CPU and OS support LASX.
+ *
+ * @note This method is only available on LoongArch 64!
+ *
+ * @return Whether the CPU and OS support LASX
+ */
++ (bool)supportsLASX;
+#endif
+
 #if defined(OF_WINDOWS) || defined(DOXYGEN)
 /**
  * @brief Returns whether the application is running on Windows NT.
